#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys
from Xlib import X, XK, display
from Xlib.ext import record
from Xlib.protocol import rq
import threading
import gi
from gi.repository import GObject, GLib
import ctypes
from ctypes import *
import capi

gdk = CDLL("libgdk-x11-2.0.so.0")
gtk = CDLL("libgtk-x11-2.0.so.0")

# python X global event， need python-xlib
class GlobalKeyBinding(GObject.GObject, threading.Thread):
    __gsignals__ = {
        'activate': (GObject.SignalFlags.RUN_LAST, None, ()),
    }

    def __init__(self):
        GObject.GObject.__init__ (self)
        threading.Thread.__init__ (self)
        self.setDaemon (True)
        self.local_dpy = display.Display()
        self.screen = self.local_dpy.screen()
        self.record_dpy = display.Display()
        self.winFlag = True

    def idle(self):
        self.emit("activate")
        return False

    def activate(self):
        GLib.idle_add(self.run)

    """
    run loop
    """
    def run(self):
        self.check_record_extension()

        # Create a recording context; we only want key and mouse events
        self.ctx = self.record_dpy.record_create_context(
            0,
            [record.AllClients],
            [{
                'core_requests': (0, 0),
                'core_replies': (0, 0),
                'ext_requests': (0, 0, 0, 0),
                'ext_replies': (0, 0, 0, 0),
                'delivered_events': (0, 0),
                'device_events': (X.KeyPress, X.MotionNotify),
                'errors': (0, 0),
                'client_started': False,
                'client_died': False,
            }])

        # Enable the context; this only returns after a call to record_disable_context,
        # while calling the callback function in the meantime
        self.record_dpy.record_enable_context(self.ctx, self.record_callback)

        # Finally free the context
        self.record_dpy.record_free_context(self.ctx)

    """
    check x extension
    """
    def check_record_extension(self):
        if not self.record_dpy.has_extension("RECORD"):
            print("RECORD extension not found")
            sys.exit(1)

    """
    lookup to keysym

    @param param1: keysym
    @return: keysym
    """
    def lookup_keysym(self, keysym):
        for name in dir(XK):
            if name[:3] == "XK_" and getattr(XK, name) == keysym:
                return name[3:]
        else:
            return keysym

    """
    x recoird callback

    @param param1: reply
    @return: no return
    """
    def record_callback(self, reply):
        if reply.category != record.FromServer:
            return
        if reply.client_swapped:
            return
        if not len(reply.data) or reply.data[0] < 2:
            return


        data = reply.data
        while len(data):
            event, data = rq.EventField(None).parse_binary_value(data, self.record_dpy.display, None, None)

            if event.type in [X.KeyPress, X.KeyRelease, X.ButtonPress, X.ButtonRelease]:
                keysym = self.local_dpy.keycode_to_keysym(event.detail, 0)

                if event.type == X.KeyPress:
                    if keysym == XK.XK_Super_L or keysym == XK.XK_Super_R:
                        print "bbbbbbbbbbbbb\n"
                        self.winFlag = False
                    else:
                        self.winFlag = True

                elif event.type == X.KeyRelease:
                    if ((keysym == XK.XK_Super_L) or (keysym == XK.XK_Super_R))  and (self.winFlag == False):
                        GLib.idle_add(self.idle)
                        print "aaaaaaaaaaaa\n"
                    else:
                        self.winFlag = True

    def set_focus_window(self, window = None):
        if self.local_dpy == None:
            return
        if window is None:
            self.window = self.screen.root
        else:
            gdk.gdk_x11_drawable_get_xid.restype = c_int
            gdk.gdk_x11_drawable_get_xid.argtypes = [ctypes.c_ulong]
            self.window = self.local_dpy.create_resource_object("window", gdk.gdk_x11_drawable_get_xid(hash(window)))
   #     self.grab(self.keytext)

